@@ -1,75 +1,195 @@
-
-#' @rdname coord_polar
+#' Polar coordinates
 #'
+#' The polar coordinate system is most commonly used for pie charts, which
+#' are a stacked bar chart in polar coordinates. \cr
+#' `r lifecycle::badge("superseded")`: `coord_polar()` has been in favour of
+#' `coord_radial()`.
+#'
+#' @param theta variable to map angle to (`x` or `y`)
+#' @param start Offset of starting point from 12 o'clock in radians. Offset
+#'   is applied clockwise or anticlockwise depending on value of `direction`.
+#' @param direction 1, clockwise; -1, anticlockwise
+#' @param clip Should drawing be clipped to the extent of the plot panel? A
+#'   setting of `"on"` (the default) means yes, and a setting of `"off"`
+#'   means no. For details, please see [`coord_cartesian()`].
 #' @param end Position from 12 o'clock in radians where plot ends, to allow
 #'   for partial polar coordinates. The default, `NULL`, is set to
 #'   `start + 2 * pi`.
-#' @param expand If `TRUE`, the default, adds a small expansion factor the
+#' @param thetalim,rlim Limits for the theta and r axes.
+#' @param expand If `TRUE`, the default, adds a small expansion factor to
 #'   the limits to prevent overlap between data and axes. If `FALSE`, limits
 #'   are taken directly from the scale.
-#' @param r_axis_inside If `TRUE`, places the radius axis inside the
-#'   panel. If `FALSE`, places the radius axis next to the panel. The default,
-#'   `NULL`, places the radius axis outside if the `start` and `end` arguments
-#'   form a full circle.
-#' @param rotate_angle If `TRUE`, transforms the `angle` aesthetic in data
+#' @param r.axis.inside One of the following:
+#'   * `NULL` (default) places the axis next to the panel if `start` and
+#'     `end` arguments form a full circle and inside the panel otherwise.
+#'   * `TRUE` to place the radius axis inside the panel.
+#'   * `FALSE` to place the radius axis next to the panel.
+#'   * A numeric value, setting a theta axis value at which
+#'     the axis should be placed inside the panel. Can be given as a length 2
+#'     vector to control primary and secondary axis placement separately.
+#' @param rotate.angle If `TRUE`, transforms the `angle` aesthetic in data
 #'   in accordance with the computed `theta` position. If `FALSE` (default),
 #'   no such transformation is performed. Can be useful to rotate text geoms in
 #'   alignment with the coordinates.
-#' @param inner.radius A `numeric` between 0 and 1 setting the size of a inner.radius hole.
-#'
+#' @param inner.radius A `numeric` between 0 and 1 setting the size of a
+#'   inner radius hole.
+#' @param reverse A string giving which directions to reverse. `"none"`
+#'   (default) keep directions as is. `"theta"` reverses the angle and `"r"`
+#'   reverses the radius. `"thetar"` reverses both the angle and the radius.
+#' @param r_axis_inside,rotate_angle `r lifecycle::badge("deprecated")`
+#' @export
 #' @note
-#' In `coord_radial()`, position guides are can be defined by using
+#' In `coord_radial()`, position guides can be defined by using
 #' `guides(r = ..., theta = ..., r.sec = ..., theta.sec = ...)`. Note that
 #' these guides require `r` and `theta` as available aesthetics. The classic
-#' `guide_axis()` can be used for the `r` positions and `guide_axis_theta()` can
+#' [guide_axis()] can be used for the `r` positions and [guide_axis_theta()] can
 #' be used for the `theta` positions. Using the `theta.sec` position is only
 #' sensible when `inner.radius > 0`.
 #'
-#' @export
+#' @seealso
+#' The `r link_book("polar coordinates section", "coord#polar-coordinates-with-coord_polar")`
 #' @examples
+#' # NOTE: Use these plots with caution - polar coordinates has
+#' # major perceptual problems.  The main point of these examples is
+#' # to demonstrate how these common plots can be described in the
+#' # grammar.  Use with EXTREME caution.
+#'
+#' # A pie chart = stacked bar chart + polar coordinates
+#' pie <- ggplot(mtcars, aes(x = factor(1), fill = factor(cyl))) +
+#'  geom_bar(width = 1)
+#' pie + coord_radial(theta = "y", expand = FALSE)
+#'
+#' \donttest{
+#'
+#' # A coxcomb plot = bar chart + polar coordinates
+#' cxc <- ggplot(mtcars, aes(x = factor(cyl))) +
+#'   geom_bar(width = 1, colour = "black")
+#' cxc + coord_radial(expand = FALSE)
+#' # A new type of plot?
+#' cxc + coord_radial(theta = "y", expand = FALSE)
+#'
+#' # The bullseye chart
+#' pie + coord_radial(expand = FALSE)
+#'
+#' # Hadley's favourite pie chart
+#' df <- data.frame(
+#'   variable = c("does not resemble", "resembles"),
+#'   value = c(20, 80)
+#' )
+#' ggplot(df, aes(x = "", y = value, fill = variable)) +
+#'   geom_col(width = 1) +
+#'   scale_fill_manual(values = c("red", "yellow")) +
+#'   coord_radial("y", start = pi / 3, expand =  FALSE) +
+#'   labs(title = "Pac man")
+#'
+#' # Windrose + doughnut plot
+#' if (require("ggplot2movies")) {
+#' movies$rrating <- cut_interval(movies$rating, length = 1)
+#' movies$budgetq <- cut_number(movies$budget, 4)
+#'
+#' doh <- ggplot(movies, aes(x = rrating, fill = budgetq))
+#'
+#' # Wind rose
+#' doh + geom_bar(width = 1) + coord_radial(expand = FALSE)
+#' # Race track plot
+#' doh + geom_bar(width = 0.9, position = "fill") +
+#'   coord_radial(theta = "y", expand = FALSE)
+#' }
+#' }
 #' # A partial polar plot
 #' ggplot(mtcars, aes(disp, mpg)) +
 #'   geom_point() +
 #'   coord_radial(start = -0.4 * pi, end = 0.4 * pi, inner.radius = 0.3)
+#'
+#' # Similar with coord_cartesian(), you can set limits.
+#' ggplot(mtcars, aes(disp, mpg)) +
+#'   geom_point() +
+#'   coord_radial(
+#'     start = -0.4 * pi,
+#'     end = 0.4 * pi, inner.radius = 0.3,
+#'     thetalim = c(200, 300),
+#'     rlim = c(15, 30),
+#'   )
 coord_radial <- function(theta = "x",
                          start = 0, end = NULL,
-                         expand = TRUE,
-                         direction = 1,
+                         thetalim = NULL, rlim = NULL, expand = TRUE,
+                         direction = deprecated(),
                          clip = "off",
-                         r_axis_inside = NULL,
-                         rotate_angle = FALSE,
-                         inner.radius = 0) {
+                         r.axis.inside = NULL,
+                         rotate.angle = FALSE,
+                         inner.radius = 0,
+                         reverse = "none",
+                         r_axis_inside = deprecated(),
+                         rotate_angle = deprecated()) {
+
+  if (lifecycle::is_present(r_axis_inside)) {
+    deprecate_warn0(
+      "3.5.1", "coord_radial(r_axis_inside)", "coord_radial(r.axis.inside)"
+    )
+    r.axis.inside <- r_axis_inside
+  }
+  if (lifecycle::is_present(rotate_angle)) {
+    deprecate_warn0(
+      "3.5.1", "coord_radial(rotate_angle)", "coord_radial(rotate.angle)"
+    )
+    rotate.angle <- rotate_angle
+  }
+  if (lifecycle::is_present(direction)) {
+    deprecate_warn0(
+      "4.0.0", "coord_radial(direction)", "coord_radial(reverse)"
+    )
+    reverse <- switch(reverse, "r" = "thetar", "theta")
+  }
 
   theta <- arg_match0(theta, c("x", "y"))
   r <- if (theta == "x") "y" else "x"
-  check_bool(r_axis_inside, allow_null = TRUE)
-  check_bool(expand)
-  check_bool(rotate_angle)
+  if (!is.numeric(r.axis.inside)) {
+    check_bool(r.axis.inside, allow_null = TRUE)
+  }
+  reverse <- arg_match0(reverse, c("theta", "thetar", "r", "none"))
+
+  check_bool(rotate.angle)
   check_number_decimal(start, allow_infinite = FALSE)
   check_number_decimal(end, allow_infinite = FALSE, allow_null = TRUE)
   check_number_decimal(inner.radius, min = 0, max = 1, allow_infinite = FALSE)
 
-  end <- end %||% (start + 2 * pi)
-  if (start > end) {
-    n_rotate <- ((start - end) %/% (2 * pi)) + 1
-    start <- start - n_rotate * 2 * pi
+  arc <- c(start, end %||% (start + 2 * pi))
+  if (arc[1] > arc[2]) {
+    n_rotate <- ((arc[1] - arc[2]) %/% (2 * pi)) + 1
+    arc[1] <- arc[1] - n_rotate * 2 * pi
   }
-  r_axis_inside <- r_axis_inside %||% !(abs(end - start) >= 1.999 * pi)
+  arc <- switch(reverse, thetar = , theta = rev(arc), arc)
+
+  r.axis.inside <- r.axis.inside %||% !(abs(arc[2] - arc[1]) >= 1.999 * pi)
+  if (isFALSE(r.axis.inside)) {
+    place <- in_arc(c(0, 0.5, 1, 1.5) * pi, arc)
+    if (!any(place)) {
+      cli::cli_warn(c(
+        "No appropriate placement found for outside {.field r.axis}.",
+        i = "Will use {.code r.axis.inside = TRUE} instead"
+      ))
+      r.axis.inside <- TRUE
+    }
+  }
+
+  inner.radius <- c(inner.radius, 1) * 0.4
+  inner.radius <- switch(reverse, thetar = , r = rev, identity)(inner.radius)
 
   ggproto(NULL, CoordRadial,
+    limits = list(theta = thetalim, r = rlim),
     theta = theta,
     r = r,
-    arc = c(start, end),
+    arc = arc,
     expand = expand,
-    direction = sign(direction),
-    r_axis_inside = r_axis_inside,
-    rotate_angle = rotate_angle,
-    inner_radius = c(inner.radius, 1) * 0.4,
+    reverse = reverse,
+    r_axis_inside = r.axis.inside,
+    rotate_angle = rotate.angle,
+    inner_radius = inner.radius,
     clip = clip
   )
 }
 
-#' @rdname ggplot2-ggproto
+#' @rdname Coord
 #' @format NULL
 #' @usage NULL
 #' @export
@@ -79,25 +199,21 @@ CoordRadial <- ggproto("CoordRadial", Coord,
     diff(details$bbox$y) / diff(details$bbox$x)
   },
 
-  is_free = function() TRUE,
+  is_free = function() {
+    TRUE
+  },
 
-  distance = function(self, x, y, details) {
+  distance = function(self, x, y, details, boost = 0.75) {
     arc <- details$arc %||% c(0, 2 * pi)
     if (self$theta == "x") {
       r <- rescale(y, from = details$r.range, to = self$inner_radius / 0.4)
-      theta <- theta_rescale_no_clip(
-        x, details$theta.range,
-        arc, self$direction
-      )
+      theta <- theta_rescale_no_clip(x, details$theta.range, arc)
     } else {
       r <- rescale(x, from = details$r.range, to = self$inner_radius / 0.4)
-      theta <- theta_rescale_no_clip(
-        y, details$theta.range,
-        arc, self$direction
-      )
+      theta <- theta_rescale_no_clip(y, details$theta.range, arc)
     }
-
-    dist_polar(r, theta)
+    # The ^boost boosts detailed munching when r is small
+    dist_polar(r^boost, theta)
   },
 
   backtransform_range = function(self, panel_params) {
@@ -114,12 +230,39 @@ CoordRadial <- ggproto("CoordRadial", Coord,
   },
 
   setup_panel_params = function(self, scale_x, scale_y, params = list()) {
-    c(
-      view_scales_polar(scale_x, self$theta, expand = self$expand),
-      view_scales_polar(scale_y, self$theta, expand = self$expand),
+    if (self$theta == "x") {
+      xlimits <- self$limits$theta
+      ylimits <- self$limits$r
+    } else {
+      xlimits <- self$limits$r
+      ylimits <- self$limits$theta
+    }
+    params <- c(
+      view_scales_polar(scale_x, self$theta, xlimits,
+        expand = params$expand[c(4, 2)]
+      ),
+      view_scales_polar(scale_y, self$theta, ylimits,
+        expand = params$expand[c(3, 1)]
+      ),
       list(bbox = polar_bbox(self$arc, inner_radius = self$inner_radius),
            arc = self$arc, inner_radius = self$inner_radius)
     )
+
+    axis_rotation <- self$r_axis_inside
+    if (is.numeric(axis_rotation)) {
+      theta_scale <- switch(self$theta, x = scale_x, y = scale_y)
+      axis_rotation <- theta_scale$transform(axis_rotation)
+      axis_rotation <- oob_squish(axis_rotation, params$theta.range)
+      axis_rotation <- theta_rescale(
+        axis_rotation, params$theta.range,
+        params$arc, 1
+      )
+      params$axis_rotation <- rep_len(axis_rotation, length.out = 2)
+    } else {
+      params$axis_rotation <- params$arc
+    }
+
+    params
   },
 
   setup_panel_guides = function(self, panel_params, guides, params = list()) {
@@ -141,7 +284,7 @@ CoordRadial <- ggproto("CoordRadial", Coord,
     # Validate appropriateness of guides
     drop_guides <- character(0)
     for (type in aesthetics) {
-      drop_guides <- check_polar_guide(drop_guides, guides, type)
+      drop_guides <- validate_polar_guide(drop_guides, guides, type)
     }
 
     guide_params <- guides$get_params(aesthetics)
@@ -151,24 +294,34 @@ CoordRadial <- ggproto("CoordRadial", Coord,
     guide_params[["theta"]]$position     <- "theta"
     guide_params[["theta.sec"]]$position <- "theta.sec"
 
-    if (self$r_axis_inside) {
+    if (self$theta == "x") {
+      opposite_r <- isTRUE(scales$r$position %in% c("top", "right"))
+    } else {
+      opposite_r <- isTRUE(scales$r$position %in% c("bottom", "left"))
+    }
+
+    if (!isFALSE(self$r_axis_inside)) {
 
-      arc <- rad2deg(self$arc)
       r_position <- c("left", "right")
-      if (self$direction == -1) {
-        arc <- rev(arc)
+      # If both opposite direction and opposite position, don't flip
+      if (xor(self$reverse %in% c("thetar", "theta"), opposite_r)) {
         r_position <- rev(r_position)
       }
-
-      guide_params[["r"]]$position     <- r_position[1]
-      guide_params[["r.sec"]]$position <- r_position[2]
+      arc <- rad2deg(panel_params$axis_rotation)
+      if (opposite_r) {
+        arc <- rev(arc)
+      }
       # Set guide text angles
       guide_params[["r"]]$angle     <- guide_params[["r"]]$angle     %|W|% arc[1]
       guide_params[["r.sec"]]$angle <- guide_params[["r.sec"]]$angle %|W|% arc[2]
     } else {
-      guide_params[["r"]]$position     <- params$r_axis
-      guide_params[["r.sec"]]$position <- opposite_position(params$r_axis)
+      r_position <- c(params$r_axis, opposite_position(params$r_axis))
+      if (opposite_r) {
+        r_position <- rev(r_position)
+      }
     }
+    guide_params[["r"]]$position     <- r_position[1]
+    guide_params[["r.sec"]]$position <- r_position[2]
 
     guide_params[drop_guides] <- list(NULL)
     guides$update_params(guide_params)
@@ -190,13 +343,20 @@ CoordRadial <- ggproto("CoordRadial", Coord,
     names(gdefs) <- aesthetics
 
     # Train theta guide
-    for (t in intersect(c("theta", "theta.sec"), aesthetics[!empty])) {
-      gdefs[[t]] <- guides[[t]]$train(gdefs[[t]], panel_params[[t]])
-      gdefs[[t]] <- guides[[t]]$transform(gdefs[[t]], self, panel_params)
-      gdefs[[t]] <- guides[[t]]$get_layer_key(gdefs[[t]], layers)
-    }
+    t <- intersect(c("theta", "theta.sec"), aesthetics[!empty])
+    gdefs[t] <- Map(
+      function(guide, guide_param, scale) {
+        guide_param$theme_suffix <- "theta"
+        guide_param <- guide$train(guide_param, scale)
+        guide_param <- guide$transform(guide_param, self, panel_params)
+        guide_param <- guide$get_layer_key(guide_param, layers)
+      },
+      guide = guides[t],
+      guide_param = gdefs[t],
+      scale = panel_params[t]
+    )
 
-    if (self$r_axis_inside) {
+    if (!isFALSE(self$r_axis_inside)) {
       # For radial axis, we need to pretend that rotation starts at 0 and
       # the bounding box is for circles, otherwise tick positions will be
       # spaced too closely.
@@ -209,32 +369,34 @@ CoordRadial <- ggproto("CoordRadial", Coord,
     temp <- modify_list(panel_params, mod)
 
     # Train radial guide
-    for (r in intersect(c("r", "r.sec"), aesthetics[!empty])) {
-      gdefs[[r]] <- guides[[r]]$train(gdefs[[r]], panel_params[[r]])
-      gdefs[[r]] <- guides[[r]]$transform(gdefs[[r]], self, temp) # Use temp
-      gdefs[[r]] <- guides[[r]]$get_layer_key(gdefs[[r]], layers)
-    }
-
-    # Set theme suffixes
-    gdefs$theta$theme_suffix     <- "theta"
-    gdefs$theta.sec$theme_suffix <- "theta"
-    gdefs$r$theme_suffix         <- "r"
-    gdefs$r.sec$theme_suffix     <- "r"
+    r <- intersect(c("r", "r.sec"), aesthetics[!empty])
+    gdefs[r] <- Map(
+      function(guide, guide_param, scale) {
+        guide_param$theme_suffix <- "r"
+        guide_param <- guide$train(guide_param, scale)
+        guide_param <- guide$transform(guide_param, self, temp)
+        guide_param <- guide$get_layer_key(guide_param, layers)
+      },
+      guide = guides[r],
+      guide_param = gdefs[r],
+      scale = panel_params[r]
+    )
 
     panel_params$guides$update_params(gdefs)
     panel_params
   },
 
   transform = function(self, data, panel_params) {
+    if (is_transform_immune(data, snake_class(self))) {
+      return(data)
+    }
+
     data <- rename_data(self, data)
     bbox <- panel_params$bbox %||% list(x = c(0, 1), y = c(0, 1))
     arc  <- panel_params$arc  %||% c(0, 2 * pi)
 
     data$r  <- r_rescale(data$r, panel_params$r.range, panel_params$inner_radius)
-    data$theta <- theta_rescale(
-      data$theta, panel_params$theta.range,
-      arc, self$direction
-    )
+    data$theta <- theta_rescale(data$theta, panel_params$theta.range, arc)
     data$x <- rescale(data$r * sin(data$theta) + 0.5, from = bbox$x)
     data$y <- rescale(data$r * cos(data$theta) + 0.5, from = bbox$y)
 
@@ -246,118 +408,82 @@ CoordRadial <- ggproto("CoordRadial", Coord,
   },
 
   render_axis_v = function(self, panel_params, theme) {
-    if (self$r_axis_inside) {
+    if (!isFALSE(self$r_axis_inside)) {
       return(list(left = zeroGrob(), right = zeroGrob()))
     }
     CoordCartesian$render_axis_v(panel_params, theme)
   },
 
   render_axis_h = function(self, panel_params, theme) {
-    if (self$r_axis_inside) {
+    if (!isFALSE(self$r_axis_inside)) {
       return(list(top = zeroGrob(), bottom = zeroGrob()))
     }
     CoordCartesian$render_axis_h(panel_params, theme)
   },
 
   render_bg = function(self, panel_params, theme) {
-
-    bbox  <- panel_params$bbox %||% list(x = c(0, 1), y = c(0, 1))
-    arc   <- panel_params$arc  %||% c(0, 2 * pi)
-    dir   <- self$direction
-    inner_radius <- panel_params$inner_radius
-
-    theta_lim <- panel_params$theta.range
-    theta_maj <- panel_params$theta.major
-    theta_min <- setdiff(panel_params$theta.minor, theta_maj)
-
-    if (length(theta_maj) > 0) {
-      theta_maj <- theta_rescale(theta_maj, theta_lim, arc, dir)
-    }
-    if (length(theta_min) > 0) {
-      theta_min <- theta_rescale(theta_min, theta_lim, arc, dir)
-    }
-    theta_fine <- seq(self$arc[1], self$arc[2], length.out = 100)
-
-    r_fine <- r_rescale(panel_params$r.major, panel_params$r.range,
-                         panel_params$inner_radius)
-
-    # This gets the proper theme element for theta and r grid lines:
-    #   panel.grid.major.x or .y
-    grid_elems <- paste(
-      c("panel.grid.major.", "panel.grid.minor.", "panel.grid.major."),
-      c(self$theta, self$theta, self$r), sep = ""
+    panel_params <- switch(
+      self$theta,
+      x = rename(panel_params, c(theta = "x", r = "y")),
+      y = rename(panel_params, c(theta = "y", r = "x"))
     )
-    grid_elems <- lapply(grid_elems, calc_element, theme = theme)
-    majortheta <- paste("panel.grid.major.", self$theta, sep = "")
-    minortheta <- paste("panel.grid.minor.", self$theta, sep = "")
-    majorr     <- paste("panel.grid.major.", self$r,     sep = "")
-
-    bg_element <- calc_element("panel.background", theme)
-    if (!inherits(bg_element, "element_blank")) {
-      background <- data_frame0(
-        x = c(Inf,  Inf, -Inf, -Inf),
-        y = c(Inf, -Inf, -Inf,  Inf)
-      )
-      background <- coord_munch(self, background, panel_params, is_closed = TRUE)
-      bg_gp <- gpar(
-        lwd = len0_null(bg_element$linewidth * .pt),
-        col = bg_element$colour, fill = bg_element$fill,
-        lty = bg_element$linetype
-      )
-      background <- polygonGrob(
-        x = background$x, y = background$y,
-        gp = bg_gp
-      )
-    } else {
-      background <- zeroGrob()
-    }
-
-    ggname("grill", grobTree(
-      background,
-      theta_grid(theta_maj, grid_elems[[1]], inner_radius, bbox),
-      theta_grid(theta_min, grid_elems[[2]], inner_radius, bbox),
-      element_render(
-        theme, majorr, name = "radius",
-        x = rescale(rep(r_fine, each = length(theta_fine)) *
-          rep(sin(theta_fine), length(r_fine)) + 0.5, from = bbox$x),
-        y = rescale(rep(r_fine, each = length(theta_fine)) *
-          rep(cos(theta_fine), length(r_fine)) + 0.5, from = bbox$y),
-        id.lengths = rep(length(theta_fine), length(r_fine)),
-        default.units = "native"
-      )
-    ))
+    guide_grid(theme, panel_params, self, square = FALSE)
   },
 
   render_fg = function(self, panel_params, theme) {
 
-    if (!self$r_axis_inside) {
+    border <- element_render(theme, "panel.border", fill = NA)
+
+    if (isFALSE(self$r_axis_inside)) {
       out <- grobTree(
         panel_guides_grob(panel_params$guides, "theta", theme),
         panel_guides_grob(panel_params$guides, "theta.sec", theme),
-        element_render(theme, "panel.border")
+        border
       )
       return(out)
     }
 
     bbox <- panel_params$bbox
     dir  <- self$direction
-    arc  <- if (dir == 1) self$arc else rev(self$arc)
-    arc  <- dir * rad2deg(-arc)
+    rot  <- panel_params$axis_rotation
+    rot  <- switch(self$reverse, thetar = , theta = rev(rot), rot)
+    rot  <- rad2deg(-rot)
 
     left <- panel_guides_grob(panel_params$guides, position = "left", theme)
-    left <- rotate_r_axis(left, arc[1], bbox, "left")
+    left <- rotate_r_axis(left, rot[1], bbox, "left")
 
     right <- panel_guides_grob(panel_params$guides, position = "right", theme)
-    right <- rotate_r_axis(right, arc[2], bbox, "right")
+    right <- rotate_r_axis(right, rot[2], bbox, "right")
 
     grobTree(
       panel_guides_grob(panel_params$guides, "theta", theme),
       panel_guides_grob(panel_params$guides, "theta.sec", theme),
       left, right,
-      element_render(theme, "panel.border")
+      border
     )
   },
 
+
+  draw_panel = function(self, panel, params, theme) {
+    clip_support <- check_device("clippingPaths", "test", maybe = TRUE)
+    if (self$clip == "on" && !isFALSE(clip_support)) {
+      clip_path <- data_frame0(
+        x = c(Inf, Inf, -Inf, -Inf),
+        y = c(Inf, -Inf, -Inf, Inf)
+      )
+      clip_path <- coord_munch(self, clip_path, params, is_closed = TRUE)
+      clip_path <- polygonGrob(clip_path$x, clip_path$y)
+      # Note that clipping path is applied to panel without coord
+      # foreground/background (added in parent method).
+      # These may contain decorations that needn't be clipped
+      panel <- list(gTree(
+        children = inject(gList(!!!panel)),
+        vp = viewport(clip = clip_path)
+      ))
+    }
+    ggproto_parent(Coord, self)$draw_panel(panel, params, theme)
+  },
+
   labels = function(self, labels, panel_params) {
     # `Layout$resolve_label()` doesn't know to look for theta/r/r.sec guides,
     # so we'll handle title propagation here.
@@ -397,31 +523,21 @@ CoordRadial <- ggproto("CoordRadial", Coord,
   },
 
   setup_params = function(self, data) {
-    if (!self$r_axis_inside) {
+    params <- ggproto_parent(Coord, self)$setup_params(data)
+    if (isFALSE(self$r_axis_inside)) {
       place <- in_arc(c(0, 0.5, 1, 1.5) * pi, self$arc)
-      if (place[1]) {
-        return(list(r_axis = "left", fake_arc = c(0, 2) * pi))
-      }
-      if (place[3]) {
-        return(list(r_axis = "left", fake_arc = c(1, 3)* pi))
-      }
-      if (place[2]) {
-        return(list(r_axis = "bottom", fake_arc = c(0.5, 2.5) * pi))
-      }
-      if (place[4]) {
-        return(list(r_axis = "bottom", fake_arc = c(1.5, 3.5) * pi))
-      }
-      cli::cli_warn(c(
-        "No appropriate placement found for {.arg r_axis_inside}.",
-        i = "Axis will be placed at panel edge."
-      ))
-      self$r_axis_inside <- TRUE
+      params$r_axis   <- if (any(place[c(1, 3)])) "left" else "bottom"
+      params$fake_arc <- switch(
+        which(place[c(1, 3, 2, 4)])[1],
+        c(0, 2), c(1, 3), c(0.5, 2.5), c(1.5, 3.5)
+      ) * pi
     }
-    return(NULL)
+    params
   }
 )
 
-view_scales_polar <- function(scale, theta = "x", expand = TRUE) {
+view_scales_polar <- function(scale, theta = "x", coord_limits = NULL,
+                              expand = TRUE) {
 
   aesthetic <- scale$aesthetics[1]
   is_theta  <- theta == aesthetic
@@ -429,7 +545,10 @@ view_scales_polar <- function(scale, theta = "x", expand = TRUE) {
 
   expansion <- default_expansion(scale, expand = expand)
   limits <- scale$get_limits()
-  continuous_range <- expand_limits_scale(scale, expansion, limits)
+  continuous_range <- expand_limits_scale(
+    scale, expansion, limits,
+    coord_limits
+  )
 
   primary <- view_scale_primary(scale, limits, continuous_range)
   view_scales <- list(
@@ -465,6 +584,8 @@ polar_bbox <- function(arc, margin = c(0.05, 0.05, 0.05, 0.05),
   if (abs(diff(arc)) >= 2 * pi) {
     return(list(x = c(0, 1), y = c(0, 1)))
   }
+  arc <- sort(arc)
+  inner_radius <- sort(inner_radius)
 
   # X and Y position of the sector arc ends
   xmax <- 0.5 * sin(arc) + 0.5
@@ -496,6 +617,11 @@ polar_bbox <- function(arc, margin = c(0.05, 0.05, 0.05, 0.05),
 # For any `theta` in [0, 2 * pi), test if theta is inside the span
 # given by `arc`
 in_arc <- function(theta, arc) {
+  # Full circle case
+  if (abs(diff(arc)) > 2 * pi - sqrt(.Machine$double.eps)) {
+    return(rep(TRUE, length(theta)))
+  }
+  # Partial circle case
   arc <- arc %% (2 * pi)
   if (arc[1] < arc[2]) {
     theta >= arc[1] & theta <= arc[2]
@@ -511,7 +637,7 @@ deg2rad <- function(deg) deg * pi / 180
 # Function to rotate a radius axis through viewport
 rotate_r_axis <- function(axis, angle, bbox, position = "left") {
 
-  if (inherits(axis, "zeroGrob")) {
+  if (is_zero(axis)) {
     return(axis)
   }
 
@@ -572,7 +698,7 @@ theta_grid <- function(theta, element, inner_radius = c(0, 0.4),
   )
 }
 
-check_polar_guide <- function(drop_list, guides, type = "theta") {
+validate_polar_guide <- function(drop_list, guides, type = "theta") {
   guide <- guides$get_guide(type)
   primary <- gsub("\\.sec$", "", type)
   if (inherits(guide, "GuideNone") || primary %in% guide$available_aes) {
