@@ -1,103 +1,16 @@
-#' Ribbons and area plots
-#'
-#' For each x value, `geom_ribbon()` displays a y interval defined
-#' by `ymin` and `ymax`. `geom_area()` is a special case of
-#' `geom_ribbon()`, where the `ymin` is fixed to 0 and `y` is used instead
-#' of `ymax`.
-#'
-#' An area plot is the continuous analogue of a stacked bar chart (see
-#' [geom_bar()]), and can be used to show how composition of the
-#' whole varies over the range of x. Choosing the order in which different
-#' components is stacked is very important, as it becomes increasing hard to
-#' see the individual pattern as you move up the stack. See
-#' [position_stack()] for the details of stacking algorithm. To facilitate
-#' stacking, the default `stat = "align"` interpolates groups to a common set
-#' of x-coordinates. To turn off this interpolation, `stat = "identity"` can
-#' be used instead.
-#'
-#' @eval rd_orientation()
-#'
-#' @eval rd_aesthetics("geom", "ribbon")
-#' @seealso
-#'   [geom_bar()] for discrete intervals (bars),
-#'   [geom_linerange()] for discrete intervals (lines),
-#'   [geom_polygon()] for general polygons
-#' @inheritParams layer
-#' @inheritParams geom_bar
-#' @param outline.type Type of the outline of the area; `"both"` draws both the
-#'   upper and lower lines, `"upper"`/`"lower"` draws the respective lines only.
-#'   `"full"` draws a closed polygon around the area.
-#' @export
-#' @examples
-#' # Generate data
-#' huron <- data.frame(year = 1875:1972, level = as.vector(LakeHuron))
-#' h <- ggplot(huron, aes(year))
-#'
-#' h + geom_ribbon(aes(ymin=0, ymax=level))
-#' h + geom_area(aes(y = level))
-#'
-#' # Orientation cannot be deduced by mapping, so must be given explicitly for
-#' # flipped orientation
-#' h + geom_area(aes(x = level, y = year), orientation = "y")
-#'
-#' # Add aesthetic mappings
-#' h +
-#'   geom_ribbon(aes(ymin = level - 1, ymax = level + 1), fill = "grey70") +
-#'   geom_line(aes(y = level))
-#'
-#' # The underlying stat_align() takes care of unaligned data points
-#' df <- data.frame(
-#'   g = c("a", "a", "a", "b", "b", "b"),
-#'   x = c(1, 3, 5, 2, 4, 6),
-#'   y = c(2, 5, 1, 3, 6, 7)
-#' )
-#' a <- ggplot(df, aes(x, y, fill = g)) +
-#'   geom_area()
-#'
-#' # Two groups have points on different X values.
-#' a + geom_point(size = 8) + facet_grid(g ~ .)
-#'
-#' # stat_align() interpolates and aligns the value so that the areas can stack
-#' # properly.
-#' a + geom_point(stat = "align", position = "stack", size = 8)
-#'
-#' # To turn off the alignment, the stat can be set to "identity"
-#' ggplot(df, aes(x, y, fill = g)) +
-#'   geom_area(stat = "identity")
-geom_ribbon <- function(mapping = NULL, data = NULL,
-                        stat = "identity", position = "identity",
-                        ...,
-                        na.rm = FALSE,
-                        orientation = NA,
-                        show.legend = NA,
-                        inherit.aes = TRUE,
-                        outline.type = "both") {
-  outline.type <- arg_match0(outline.type, c("both", "upper", "lower", "full"))
-
-  layer(
-    data = data,
-    mapping = mapping,
-    stat = stat,
-    geom = GeomRibbon,
-    position = position,
-    show.legend = show.legend,
-    inherit.aes = inherit.aes,
-    params = list2(
-      na.rm = na.rm,
-      orientation = orientation,
-      outline.type = outline.type,
-      ...
-    )
-  )
-}
-
-#' @rdname ggplot2-ggproto
+#' @rdname Geom
 #' @format NULL
 #' @usage NULL
 #' @export
 GeomRibbon <- ggproto("GeomRibbon", Geom,
-  default_aes = aes(colour = NA, fill = "grey20", linewidth = 0.5, linetype = 1,
-    alpha = NA),
+
+  default_aes = aes(
+    colour = from_theme(colour %||% NA),
+    fill = from_theme(fill %||% col_mix(ink, paper, 0.2)),
+    linewidth = from_theme(borderwidth),
+    linetype = from_theme(bordertype),
+    alpha = NA
+  ),
 
   required_aes = c("x|y", "ymin|xmin", "ymax|xmax"),
 
@@ -122,24 +35,80 @@ GeomRibbon <- ggproto("GeomRibbon", Geom,
 
   draw_key = draw_key_polygon,
 
-  handle_na = function(data, params) {
+  handle_na = function(self, data, params) {
+
+    vars <- vapply(
+      strsplit(self$required_aes, "|", fixed = TRUE),
+      `[[`, i = 1, character(1)
+    )
+    if (isTRUE(params$flipped_aes || any(data$flipped_aes) %||% FALSE)) {
+      vars <- switch_orientation(vars)
+    }
+    vars <- c(vars, self$non_missing_aes)
+
+    missing <- detect_missing(data, vars, finite = FALSE)
+    if (!any(missing)) {
+      return(data)
+    }
+    # We're rearranging groups to account for missing values
+    data$group <- vec_identify_runs(data_frame0(missing, data$group))
+    data <- vec_slice(data, !missing)
+
+    if (!params$na.rm) {
+      cli::cli_warn(
+        "Removed {sum(missing)} row{?s} containing missing values or values \\
+          outside the scale range ({.fn {snake_class(self)}})."
+      )
+    }
     data
   },
 
   draw_group = function(self, data, panel_params, coord, lineend = "butt",
                         linejoin = "round", linemitre = 10, na.rm = FALSE,
                         flipped_aes = FALSE, outline.type = "both") {
-    data <- check_linewidth(data, snake_class(self))
+    data <- fix_linewidth(data, snake_class(self))
     data <- flip_data(data, flipped_aes)
-    if (na.rm) data <- data[stats::complete.cases(data[c("x", "ymin", "ymax")]), ]
     data <- data[order(data$group), ]
 
     # Check that aesthetics are constant
-    aes <- unique0(data[names(data) %in% c("colour", "fill", "linewidth", "linetype", "alpha")])
-    if (nrow(aes) > 1) {
-      cli::cli_abort("Aesthetics can not vary along a ribbon.")
+    aes <- lapply(
+      data[names(data) %in% c("colour", "fill", "linewidth", "linetype", "alpha")],
+      unique0
+    )
+    non_constant <- names(aes)[lengths(aes) > 1]
+    if (coord$is_linear()) {
+      if (any(c("fill", "alpha") %in% non_constant)) {
+        check_device("gradients", action = "abort", maybe = TRUE)
+      }
+      # For linear coords, we can make a fill/alpha gradient, so we allow
+      # these to vary
+      non_constant <- setdiff(non_constant, c("fill", "alpha"))
+    }
+    if (length(non_constant) > 0) {
+      cli::cli_abort(
+        "Aesthetics can not vary along a ribbon: {.and {.field {non_constant}}}."
+      )
+    }
+    if ((length(aes$fill) > 1 || length(aes$alpha) > 1)) {
+      transformed <- coord$transform(flip_data(data, flipped_aes), panel_params)
+      if (flipped_aes) {
+        keep <- is.finite(transformed$y)
+        args <- list(
+          colours = alpha(data$fill, data$alpha)[keep],
+          stops = rescale(transformed$y)[keep],
+          y1 = 0, y2 = 1, x1 = 0.5, x2 = 0.5
+        )
+      } else {
+        keep <- is.finite(transformed$x)
+        args <- list(
+          colours = alpha(data$fill, data$alpha)[keep],
+          stops = rescale(transformed$x)[keep],
+          x1 = 0, x2 = 1, y1 = 0.5, y2 = 0.5
+        )
+      }
+      aes$fill <- inject(linearGradient(!!!args))
+      aes$alpha <- NA
     }
-    aes <- as.list(aes)
 
     # Instead of removing NA values from the data and plotting a single
     # polygon, we want to "stop" plotting the polygon whenever we're
@@ -182,10 +151,10 @@ GeomRibbon <- ggproto("GeomRibbon", Geom,
     g_poly <- polygonGrob(
       munched_poly$x, munched_poly$y, id = munched_poly$id,
       default.units = "native",
-      gp = gpar(
+      gp = gg_par(
         fill = fill_alpha(aes$fill, aes$alpha),
         col = if (is_full_outline) aes$colour else NA,
-        lwd = if (is_full_outline) aes$linewidth * .pt else 0,
+        lwd = if (is_full_outline) aes$linewidth else 0,
         lty = if (is_full_outline) aes$linetype else 1,
         lineend = lineend,
         linejoin = linejoin,
@@ -213,9 +182,9 @@ GeomRibbon <- ggproto("GeomRibbon", Geom,
     g_lines <- polylineGrob(
       munched_lines$x, munched_lines$y, id = munched_lines$id,
       default.units = "native",
-      gp = gpar(
+      gp = gg_par(
         col = aes$colour,
-        lwd = aes$linewidth * .pt,
+        lwd = aes$linewidth,
         lty = aes$linetype,
         lineend = lineend,
         linejoin = linejoin,
@@ -229,38 +198,11 @@ GeomRibbon <- ggproto("GeomRibbon", Geom,
   rename_size = TRUE
 )
 
-#' @rdname geom_ribbon
-#' @export
-geom_area <- function(mapping = NULL, data = NULL, stat = "align",
-                      position = "stack", na.rm = FALSE, orientation = NA,
-                      show.legend = NA, inherit.aes = TRUE, ...,
-                      outline.type = "upper") {
-  outline.type <- arg_match0(outline.type, c("both", "upper", "lower", "full"))
-
-  layer(
-    data = data,
-    mapping = mapping,
-    stat = stat,
-    geom = GeomArea,
-    position = position,
-    show.legend = show.legend,
-    inherit.aes = inherit.aes,
-    params = list2(
-      na.rm = na.rm,
-      orientation = orientation,
-      outline.type = outline.type,
-      ...
-    )
-  )
-}
-
-#' @rdname ggplot2-ggproto
+#' @rdname Geom
 #' @format NULL
 #' @usage NULL
 #' @export
 GeomArea <- ggproto("GeomArea", GeomRibbon,
-  default_aes = aes(colour = NA, fill = "grey20", linewidth = 0.5, linetype = 1,
-    alpha = NA),
 
   required_aes = c("x", "y"),
 
@@ -276,3 +218,88 @@ GeomArea <- ggproto("GeomArea", GeomRibbon,
     flip_data(data, params$flipped_aes)
   }
 )
+
+#' Ribbons and area plots
+#'
+#' For each x value, `geom_ribbon()` displays a y interval defined
+#' by `ymin` and `ymax`. `geom_area()` is a special case of
+#' `geom_ribbon()`, where the `ymin` is fixed to 0 and `y` is used instead
+#' of `ymax`.
+#'
+#' An area plot is the continuous analogue of a stacked bar chart (see
+#' [geom_bar()]), and can be used to show how composition of the
+#' whole varies over the range of x. Choosing the order in which different
+#' components is stacked is very important, as it becomes increasing hard to
+#' see the individual pattern as you move up the stack. See
+#' [position_stack()] for the details of stacking algorithm. To facilitate
+#' stacking, the default `stat = "align"` interpolates groups to a common set
+#' of x-coordinates. To turn off this interpolation, `stat = "identity"` can
+#' be used instead.
+#'
+#' @eval rd_orientation()
+#'
+#' @aesthetics GeomRibbon
+#' @seealso
+#'   [geom_bar()] for discrete intervals (bars),
+#'   [geom_linerange()] for discrete intervals (lines),
+#'   [geom_polygon()] for general polygons
+#' @inheritParams layer
+#' @inheritParams geom_bar
+#' @param linemitre Line mitre limit (number greater than 1).
+#' @param outline.type Type of the outline of the area; `"both"` draws both the
+#'   upper and lower lines, `"upper"`/`"lower"` draws the respective lines only.
+#'   `"full"` draws a closed polygon around the area.
+#' @export
+#' @examples
+#' # Generate data
+#' huron <- data.frame(year = 1875:1972, level = as.vector(LakeHuron))
+#' h <- ggplot(huron, aes(year))
+#'
+#' h + geom_ribbon(aes(ymin=0, ymax=level))
+#' h + geom_area(aes(y = level))
+#'
+#' # Orientation cannot be deduced by mapping, so must be given explicitly for
+#' # flipped orientation
+#' h + geom_area(aes(x = level, y = year), orientation = "y")
+#'
+#' # Add aesthetic mappings
+#' h +
+#'   geom_ribbon(aes(ymin = level - 1, ymax = level + 1), fill = "grey70") +
+#'   geom_line(aes(y = level))
+#'
+#' # The underlying stat_align() takes care of unaligned data points
+#' df <- data.frame(
+#'   g = c("a", "a", "a", "b", "b", "b"),
+#'   x = c(1, 3, 5, 2, 4, 6),
+#'   y = c(2, 5, 1, 3, 6, 7)
+#' )
+#' a <- ggplot(df, aes(x, y, fill = g)) +
+#'   geom_area()
+#'
+#' # Two groups have points on different X values.
+#' a + geom_point(size = 8) + facet_grid(g ~ .)
+#'
+#' # stat_align() interpolates and aligns the value so that the areas can stack
+#' # properly.
+#' a + geom_point(stat = "align", position = "stack", size = 8)
+#'
+#' # To turn off the alignment, the stat can be set to "identity"
+#' ggplot(df, aes(x, y, fill = g)) +
+#'   geom_area(stat = "identity")
+#'
+geom_ribbon <- make_constructor(
+  GeomRibbon, orientation = NA,
+  checks = exprs(
+    outline.type <- arg_match0(outline.type, c("both", "upper", "lower", "full"))
+  )
+)
+
+#' @rdname geom_ribbon
+#' @export
+geom_area <- make_constructor(
+  GeomArea, stat = "align", position = "stack",
+  orientation = NA, outline.type = "upper",
+  checks = exprs(
+    outline.type <- arg_match0(outline.type, c("both", "upper", "lower", "full"))
+  )
+)
