@@ -1,3 +1,55 @@
+#' @rdname Geom
+#' @format NULL
+#' @usage NULL
+#' @export
+GeomText <- ggproto(
+  "GeomText", Geom,
+  required_aes = c("x", "y", "label"),
+
+  non_missing_aes = "angle",
+
+  default_aes = aes(
+    colour = from_theme(colour %||% ink),
+    family = from_theme(family),
+    size = from_theme(fontsize),
+    angle = 0, hjust = 0.5,
+    vjust = 0.5, alpha = NA, fontface = 1, lineheight = 1.2
+  ),
+
+  draw_panel = function(data, panel_params, coord, parse = FALSE,
+                        na.rm = FALSE, check_overlap = FALSE,
+                        size.unit = "mm") {
+    lab <- data$label
+    if (parse) {
+      lab <- parse_safe(as.character(lab))
+    }
+
+    data <- coord$transform(data, panel_params)
+
+    data$vjust <- compute_just(data$vjust, data$y, data$x, data$angle)
+    data$hjust <- compute_just(data$hjust, data$x, data$y, data$angle)
+
+    size.unit <- resolve_text_unit(size.unit)
+
+    textGrob(
+      lab,
+      data$x, data$y, default.units = "native",
+      hjust = data$hjust, vjust = data$vjust,
+      rot = data$angle,
+      gp = gg_par(
+        col = alpha(data$colour, data$alpha),
+        fontsize = data$size * size.unit,
+        fontfamily = data$family,
+        fontface = data$fontface,
+        lineheight = data$lineheight
+      ),
+      check.overlap = check_overlap
+    )
+  },
+
+  draw_key = draw_key_text
+)
+
 #' Text
 #'
 #' Text geoms are useful for labeling plots. They can be used by themselves as
@@ -23,7 +75,7 @@
 #' \href{https://cran.r-project.org/package=ggrepel}{ggrepel}
 #' package.
 #'
-#' @eval rd_aesthetics("geom", "text")
+#' @aesthetics GeomText
 #' @section `geom_label()`:
 #' Currently `geom_label()` does not support the `check_overlap` argument. Also,
 #' it is considerably slower than `geom_text()`. The `fill` aesthetic controls
@@ -31,8 +83,8 @@
 #'
 #' @section Alignment:
 #' You can modify text alignment with the `vjust` and `hjust`
-#' aesthetics. These can either be a number between 0 (right/bottom) and
-#' 1 (top/left) or a character (`"left"`, `"middle"`, `"right"`, `"bottom"`,
+#' aesthetics. These can either be a number between 0 (left/bottom) and
+#' 1 (right/top) or a character (`"left"`, `"middle"`, `"right"`, `"bottom"`,
 #' `"center"`, `"top"`). There are two special alignments: `"inward"` and
 #' `"outward"`. Inward always aligns text towards the center, and outward
 #' aligns it away from the center.
@@ -41,12 +93,6 @@
 #' @inheritParams geom_point
 #' @param parse If `TRUE`, the labels will be parsed into expressions and
 #'   displayed as described in `?plotmath`.
-#' @param nudge_x,nudge_y Horizontal and vertical adjustment to nudge labels by.
-#'   Useful for offsetting text from points, particularly on discrete scales.
-#'   Cannot be jointly specified with `position`.
-#' @param position Position adjustment, either as a string, or the result of
-#'  a call to a position adjustment function. Cannot be jointly specified with
-#'  `nudge_x` or `nudge_y`.
 #' @param check_overlap If `TRUE`, text that overlaps previous text in the
 #'   same layer will not be plotted. `check_overlap` happens at draw time and in
 #'   the order of the data. Therefore data should be arranged by the label
@@ -56,6 +102,8 @@
 #'   (`"mm"`, default), points (`"pt"`), centimetres (`"cm"`), inches (`"in"`),
 #'   or picas (`"pc"`).
 #' @export
+#' @seealso
+#' The `r link_book("text labels section", "annotations#sec-text-labels")`
 #' @examples
 #' p <- ggplot(mtcars, aes(wt, mpg, label = rownames(mtcars)))
 #'
@@ -86,13 +134,14 @@
 #' # Add aesthetic mappings
 #' p + geom_text(aes(colour = factor(cyl)))
 #' p + geom_text(aes(colour = factor(cyl))) +
-#'   scale_colour_discrete(l = 40)
+#'   scale_colour_hue(l = 40)
 #' p + geom_label(aes(fill = factor(cyl)), colour = "white", fontface = "bold")
 #'
-#' p + geom_text(aes(size = wt))
+#' # Scale size of text, and change legend key glyph from a to point
+#' p + geom_text(aes(size = wt), key_glyph = "point")
 #' # Scale height of text, rather than sqrt(height)
 #' p +
-#'   geom_text(aes(size = wt)) +
+#'   geom_text(aes(size = wt), key_glyph = "point") +
 #'   scale_radius(range = c(3,6))
 #'
 #' # You can display expressions by setting parse = TRUE.  The
@@ -155,94 +204,7 @@
 #' ggplot(df, aes(x, y)) +
 #'   geom_text(aes(label = text), vjust = "inward", hjust = "inward")
 #' }
-geom_text <- function(mapping = NULL, data = NULL,
-                      stat = "identity", position = "identity",
-                      ...,
-                      parse = FALSE,
-                      nudge_x = 0,
-                      nudge_y = 0,
-                      check_overlap = FALSE,
-                      size.unit = "mm",
-                      na.rm = FALSE,
-                      show.legend = NA,
-                      inherit.aes = TRUE)
-{
-  if (!missing(nudge_x) || !missing(nudge_y)) {
-    if (!missing(position)) {
-      cli::cli_abort(c(
-        "Both {.arg position} and {.arg nudge_x}/{.arg nudge_y} are supplied.",
-        "i" = "Only use one approach to alter the position."
-      ))
-    }
-
-    position <- position_nudge(nudge_x, nudge_y)
-  }
-
-  layer(
-    data = data,
-    mapping = mapping,
-    stat = stat,
-    geom = GeomText,
-    position = position,
-    show.legend = show.legend,
-    inherit.aes = inherit.aes,
-    params = list2(
-      parse = parse,
-      check_overlap = check_overlap,
-      size.unit = size.unit,
-      na.rm = na.rm,
-      ...
-    )
-  )
-}
-
-#' @rdname ggplot2-ggproto
-#' @format NULL
-#' @usage NULL
-#' @export
-GeomText <- ggproto("GeomText", Geom,
-  required_aes = c("x", "y", "label"),
-
-  non_missing_aes = "angle",
-
-  default_aes = aes(
-    colour = "black", size = 3.88, angle = 0, hjust = 0.5,
-    vjust = 0.5, alpha = NA, family = "", fontface = 1, lineheight = 1.2
-  ),
-
-  draw_panel = function(data, panel_params, coord, parse = FALSE,
-                        na.rm = FALSE, check_overlap = FALSE,
-                        size.unit = "mm") {
-    lab <- data$label
-    if (parse) {
-      lab <- parse_safe(as.character(lab))
-    }
-
-    data <- coord$transform(data, panel_params)
-
-    data$vjust <- compute_just(data$vjust, data$y, data$x, data$angle)
-    data$hjust <- compute_just(data$hjust, data$x, data$y, data$angle)
-
-    size.unit <- resolve_text_unit(size.unit)
-
-    textGrob(
-      lab,
-      data$x, data$y, default.units = "native",
-      hjust = data$hjust, vjust = data$vjust,
-      rot = data$angle,
-      gp = gpar(
-        col = alpha(data$colour, data$alpha),
-        fontsize = data$size * size.unit,
-        fontfamily = data$family,
-        fontface = data$fontface,
-        lineheight = data$lineheight
-      ),
-      check.overlap = check_overlap
-    )
-  },
-
-  draw_key = draw_key_text
-)
+geom_text <- make_constructor(GeomText, position = "nudge")
 
 compute_just <- function(just, a = 0.5, b = a, angle = 0) {
   if (!is.character(just)) {
