@@ -6,6 +6,9 @@
 #' [scale_(x|y)_discrete()][scale_x_discrete()].
 #'
 #' @inheritParams guide_legend
+#' @param theme A [`theme`][theme()] object to style the guide individually or
+#'   differently from the plot's theme settings. The `theme` argument in the
+#'   guide partially overrides, and is combined with, the plot's theme.
 #' @param check.overlap silently remove overlapping labels,
 #'   (recursively) prioritizing the first, last, and middle labels.
 #' @param angle Compared to setting the angle in [theme()] / [element_text()],
@@ -78,7 +81,7 @@ guide_axis <- function(title = waiver(), theme = NULL, check.overlap = FALSE,
   )
 }
 
-#' @rdname ggplot2-ggproto
+#' @rdname Guide
 #' @format NULL
 #' @usage NULL
 #' @export
@@ -115,6 +118,9 @@ GuideAxis <- ggproto(
 
   extract_key = function(scale, aesthetic, minor.ticks = FALSE, ...) {
     major <- Guide$extract_key(scale, aesthetic, ...)
+    if (is.null(major) && is.null(scale$scale$get_breaks())) {
+      major <- data_frame0()
+    }
     if (!minor.ticks) {
       return(major)
     }
@@ -133,6 +139,11 @@ GuideAxis <- ggproto(
 
     if (nrow(major) > 0) {
       major$.type <- "major"
+      if (!vec_is(minor$.value, major$.value)) {
+        # If we have mixed types of values, which may happen in discrete scales,
+        # discard minor values in favour of the major values.
+        minor$.value <- NULL
+      }
       vec_rbind(major, minor)
     } else {
       minor
@@ -147,47 +158,58 @@ GuideAxis <- ggproto(
   extract_decor = function(scale, aesthetic, position, key, cap = "none", ...) {
 
     value <- c(-Inf, Inf)
-    if (cap %in% c("both", "upper")) {
+    has_key <- !(is.null(key) || nrow(key) < 1)
+    if (cap %in% c("both", "upper") && has_key) {
       value[2] <- max(key[[aesthetic]])
     }
-    if (cap %in% c("both", "lower")) {
+    if (cap %in% c("both", "lower") && has_key) {
       value[1] <- min(key[[aesthetic]])
     }
 
     opposite <- setdiff(c("x", "y"), aesthetic)
-    opposite_value <- if (position %in% c("top", "right")) -Inf else Inf
 
-    data_frame(
-      !!aesthetic := value,
-      !!opposite  := opposite_value
-    )
+    data_frame(!!aesthetic := value)
   },
 
   transform = function(self, params, coord, panel_params) {
     key <- params$key
     position <- params$position
+    check <- FALSE
 
-    if (is.null(position) || nrow(key) == 0) {
+    aesthetic <- names(key)[!grepl("^\\.", names(key))]
+    ortho     <- setdiff(c("x", "y"), params$aesthetic)
+    override  <- switch(position %||% "", bottom = , left = -Inf, Inf)
+
+    if (!(panel_params$reverse %||% "none") %in% c("xy", ortho)) {
+      override <- -override
+    }
+
+    if (!(is.null(position) || nrow(key) == 0)) {
+      check <- TRUE
+      if (!all(c("x", "y") %in% aesthetic)) {
+        key[[ortho]] <- override
+      }
+      key <- coord$transform(key, panel_params)
+      params$key <- key
+    }
+
+    if (!is.null(params$decor)) {
+      if (!ortho %in% names(params$decor)) {
+        params$decor[[ortho]] <- override
+      }
+      params$decor <- coord_munch(coord, params$decor, panel_params)
+
+      if (!coord$is_linear()) {
+        # For non-linear coords, we hardcode the opposite position
+        params$decor$x <- switch(position, left = 1, right = 0, params$decor$x)
+        params$decor$y <- switch(position, top = 0, bottom = 1, params$decor$y)
+      }
+    }
+
+    if (!check) {
       return(params)
     }
 
-    aesthetics <- names(key)[!grepl("^\\.", names(key))]
-    if (!all(c("x", "y") %in% aesthetics)) {
-      other_aesthetic <- setdiff(c("x", "y"), aesthetics)
-      override_value <- if (position %in% c("bottom", "left")) -Inf else Inf
-      key[[other_aesthetic]] <- override_value
-    }
-    key <- coord$transform(key, panel_params)
-    params$key <- key
-
-    params$decor <- coord_munch(coord, params$decor, panel_params)
-
-    if (!coord$is_linear()) {
-      # For non-linear coords, we hardcode the opposite position
-      params$decor$x <- switch(position, left = 1, right = 0, params$decor$x)
-      params$decor$y <- switch(position, top = 0, bottom = 1, params$decor$y)
-    }
-
     # Ported over from `warn_for_position_guide`
     # This is trying to catch when a user specifies a position perpendicular
     # to the direction of the axis (e.g., a "y" axis on "top").
@@ -239,21 +261,14 @@ GuideAxis <- ggproto(
   },
 
   override_elements = function(params, elements, theme) {
-    label <- elements$text
-    if (!is_theme_element(label, "text")) {
-      return(elements)
+    elements$text <-
+      label_angle_heuristic(elements$text, params$position, params$angle)
+    if (is_theme_element(elements$ticks, "blank")) {
+      elements$major_length <- unit(0, "cm")
+    }
+    if (is_theme_element(elements$minor, "blank") || isFALSE(params$minor.ticks)) {
+      elements$minor_length <- unit(0, "cm")
     }
-    label_overrides <- axis_label_element_overrides(
-      params$position, params$angle
-    )
-    # label_overrides is an element_text, but label_element may not be;
-    # to merge the two elements, we just copy angle, hjust, and vjust
-    # unless their values are NULL
-    label$angle <- label_overrides$angle %||% label$angle
-    label$hjust <- label_overrides$hjust %||% label$hjust
-    label$vjust <- label_overrides$vjust %||% label$vjust
-
-    elements$text <- label
     return(elements)
   },
 
@@ -368,7 +383,7 @@ GuideAxis <- ggproto(
     # Ticks
     major_cm <- convertUnit(elements$major_length, "cm", valueOnly = TRUE)
     range <- range(0, major_cm)
-    if (params$minor.ticks && !inherits(elements$minor, "element_blank")) {
+    if (params$minor.ticks && !is_theme_element(elements$minor, "blank")) {
       minor_cm <- convertUnit(elements$minor_length, "cm", valueOnly = TRUE)
       range <- range(range, minor_cm)
     }
@@ -409,6 +424,7 @@ GuideAxis <- ggproto(
     # Unlist the 'label' grobs
     z <- if (params$position == "left") c(2, 1, 3) else 1:3
     z <- rep(z, c(1, length(grobs$labels), 1))
+    has_labels <- !is_zero(grobs$labels[[1]])
     grobs  <- c(list(grobs$ticks), grobs$labels, list(grobs$title))
 
     # Initialise empty gtable
@@ -430,10 +446,25 @@ GuideAxis <- ggproto(
     vp <- exec(
       viewport,
       !!params$orth_aes := unit(params$orth_side, "npc"),
-      !!params$orth_size := params$measure_gtable(gt),
+      !!params$orth_size := max(params$measure_gtable(gt), unit(1, "npc")),
       just = params$opposite
     )
 
+    # Add null-unit padding to justify based on eventual gtable cell shape
+    # rather than dimensions of this axis alone.
+    if (has_labels && params$position %in% c("left", "right")) {
+      where <- layout$l[-c(1, length(layout$l))]
+      just <- rotate_just(element = elements$text)$hjust %||% 0.5
+      gt <- gtable_add_cols(gt, unit(just, "null"), pos = min(where) - 1)
+      gt <- gtable_add_cols(gt, unit(1 - just, "null"), pos = max(where) + 1)
+    }
+    if (has_labels && params$position %in% c("top", "bottom")) {
+      where <- layout$t[-c(1, length(layout$t))]
+      just <- rotate_just(element = elements$text)$vjust %||% 0.5
+      gt <- gtable_add_rows(gt, unit(1 - just, "null"), pos = min(where) - 1)
+      gt <- gtable_add_rows(gt, unit(just, "null"), pos = max(where) + 1)
+    }
+
     # Assemble with axis line
     absoluteGrob(
       gList(axis_line, gt),
@@ -553,49 +584,40 @@ axis_label_priority_between <- function(x, y) {
   )
 }
 
-#' Override axis text angle and alignment
+#' Override text angle and alignment
 #'
+#' @param element An `element_text()`
 #' @param axis_position One of bottom, left, top, or right
 #' @param angle The text angle, or NULL to override nothing
 #'
 #' @return An [element_text()] that contains parameters that should be
 #'   overridden from the user- or theme-supplied element.
 #' @noRd
-#'
-axis_label_element_overrides <- function(axis_position, angle = NULL) {
-
-  if (is.null(angle) || is.waive(angle)) {
-    return(element_text(angle = NULL, hjust = NULL, vjust = NULL))
+label_angle_heuristic <- function(element, position, angle) {
+  if (!is_theme_element(element, "text")
+      || is.null(position)
+      || is.null(angle %|W|% NULL)) {
+    return(element)
   }
+  arg_match0(position, .trbl)
 
   check_number_decimal(angle)
-  angle <- angle %% 360
-  arg_match0(
-    axis_position,
-    c("bottom", "left", "top", "right")
-  )
+  radian <- deg2rad(angle)
+  digits <- 3
 
-  if (axis_position == "bottom") {
+  # Taking the sign of the (co)sine snaps the value to c(-1, 0, 1)
+  # Doing `x / 2 + 0.5` rescales it to c(0, 0.5, 1), which are good values for justification
+  # The rounding step ensures we can get (co)sine to exact 0 so it can become 0.5
+  # which we need for center-justifications
+  cosine <- sign(round(cos(radian), digits)) / 2 + 0.5
+  sine   <- sign(round(sin(radian), digits)) / 2 + 0.5
 
-    hjust = if (angle %in% c(0, 180))  0.5 else if (angle < 180) 1 else 0
-    vjust = if (angle %in% c(90, 270)) 0.5 else if (angle > 90 & angle < 270) 0 else 1
+  # Depending on position, we might need to swap or flip justification values
+  hjust <- switch(position, left = cosine, right = 1 - cosine, top = 1 - sine, sine)
+  vjust <- switch(position, left = 1 - sine, right = sine, top = 1 - cosine, cosine)
 
-  } else if (axis_position == "left") {
-
-    hjust = if (angle %in% c(90, 270)) 0.5 else if (angle > 90 & angle < 270) 0 else 1
-    vjust = if (angle %in% c(0, 180))  0.5 else if (angle < 180) 0 else 1
-
-  } else if (axis_position == "top") {
-
-    hjust = if (angle %in% c(0, 180))  0.5 else if (angle < 180) 0 else 1
-    vjust = if (angle %in% c(90, 270)) 0.5 else if (angle > 90 & angle < 270) 1 else 0
-
-  } else if (axis_position == "right") {
-
-    hjust = if (angle %in% c(90, 270)) 0.5 else if (angle > 90 & angle < 270) 1 else 0
-    vjust = if (angle %in% c(0, 180))  0.5 else if (angle < 180) 1 else 0
-
-  }
-
-  element_text(angle = angle, hjust = hjust, vjust = vjust)
+  element@angle <- angle %||% try_prop(element, "angle")
+  element@hjust <- hjust %||% try_prop(element, "hjust")
+  element@vjust <- vjust %||% try_prop(element, "vjust")
+  element
 }
