@@ -103,56 +103,51 @@
 #'     mapping = aes(x = group, y = group_mean), data = group_means_df,
 #'     colour = 'red', size = 3
 #'   )
-ggplot <- function(data = NULL, mapping = aes(), ...,
-                   environment = parent.frame()) {
+ggplot <- function(
+  data = NULL,
+  mapping = aes(),
+  ...,
+  environment = parent.frame()
+) {
   UseMethod("ggplot")
 }
 
 #' @export
-ggplot.default <- function(data = NULL, mapping = aes(), ...,
-                           environment = parent.frame()) {
-  if (!missing(mapping) && !is_mapping(mapping)) {
-    cli::cli_abort(c(
-      "{.arg mapping} must be created with {.fn aes}.",
-      "x" = "You've supplied {.obj_type_friendly {mapping}}."
-    ))
+ggplot.default <-
+  function(data, mapping = aes(), ..., environment = parent.frame()) {
+
+  if (!missing(mapping)) {
+    mapping <- validate_mapping(mapping)
+  }
+  if (missing(data)) {
+    data <- NULL
   }
 
   data <- fortify(data, ...)
 
-  p <- structure(list(
+  p <- class_ggplot(
     data = data,
-    layers = list(),
-    scales = scales_list(),
-    guides = guides_list(),
     mapping = mapping,
-    theme = list(),
-    coordinates = coord_cartesian(default = TRUE),
-    facet = facet_null(),
-    plot_env = environment,
-    layout = ggproto(NULL, Layout)
-  ), class = c("gg", "ggplot"))
-
-  p$labels <- make_labels(mapping)
+    plot_env = environment
+  )
+  class(p) <- union(union(c("ggplot2::ggplot", "ggplot"), class(p)), "gg")
 
   set_last_plot(p)
   p
 }
 
 #' @export
-ggplot.function <- function(data = NULL, mapping = aes(), ...,
-                            environment = parent.frame()) {
-  # Added to avoid functions end in ggplot.default
-  cli::cli_abort(c(
-    "{.arg data} cannot be a function.",
-    "i" = "Have you misspelled the {.arg data} argument in {.fn ggplot}"
-  ))
-}
+ggplot.function <- function(data, ...) {
+    # Added to avoid functions end in ggplot.default
+    cli::cli_abort(c(
+      "{.arg data} cannot be a function.",
+      "i" = "Have you misspelled the {.arg data} argument in {.fn ggplot}?"
+    ))
+  }
 
 plot_clone <- function(plot) {
   p <- plot
-  p$scales <- plot$scales$clone()
-
+  p@scales <- plot@scales$clone()
   p
 }
 
@@ -161,7 +156,7 @@ plot_clone <- function(plot) {
 #' @keywords internal
 #' @export
 #' @name is_tests
-is_ggplot <- function(x) inherits(x, "ggplot")
+is_ggplot <- function(x) S7::S7_inherits(x, class_ggplot)
 
 #' @export
 #' @rdname is_tests
@@ -184,52 +179,104 @@ is.ggplot <- function(x) {
 #' @param ... other arguments not used by this method
 #' @keywords hplot
 #' @return Invisibly returns the original plot.
-#' @export
-#' @method print ggplot
+#' @name print.ggplot
+#' @aliases plot.ggplot
 #' @examples
-#' colours <- list(~class, ~drv, ~fl)
+#' colours <- c("class", "drv", "fl")
 #'
 #' # Doesn't seem to do anything!
 #' for (colour in colours) {
-#'   ggplot(mpg, aes_(~ displ, ~ hwy, colour = colour)) +
+#'   ggplot(mpg, aes(displ, hwy, colour = .data[[colour]])) +
 #'     geom_point()
 #' }
 #'
-#' # Works when we explicitly print the plots
 #' for (colour in colours) {
-#'   print(ggplot(mpg, aes_(~ displ, ~ hwy, colour = colour)) +
-#'     geom_point())
+#'   print(ggplot(mpg, aes(displ, hwy, colour = .data[[colour]])) +
+#'           geom_point())
 #' }
-print.ggplot <- function(x, newpage = is.null(vp), vp = NULL, ...) {
-  set_last_plot(x)
-  if (newpage) grid.newpage()
+local({
+  S7::method(print, class_ggplot) <- S7::method(plot, class_ggplot) <-
+    function(x, newpage = is.null(vp), vp = NULL, ...) {
+      set_last_plot(x)
+      if (newpage) grid.newpage()
 
-  # Record dependency on 'ggplot2' on the display list
-  # (AFTER grid.newpage())
-  grDevices::recordGraphics(
-    requireNamespace("ggplot2", quietly = TRUE),
-    list(),
-    getNamespace("ggplot2")
-  )
+      # Record dependency on 'ggplot2' on the display list
+      # (AFTER grid.newpage())
+      grDevices::recordGraphics(
+        requireNamespace("ggplot2", quietly = TRUE),
+        list(),
+        getNamespace("ggplot2")
+      )
 
-  data <- ggplot_build(x)
+      data <- ggplot_build(x)
 
-  gtable <- ggplot_gtable(data)
-  if (is.null(vp)) {
-    grid.draw(gtable)
-  } else {
-    if (is.character(vp)) seekViewport(vp) else pushViewport(vp)
-    grid.draw(gtable)
-    upViewport()
+      gtable <- ggplot_gtable(data)
+      if (is.null(vp)) {
+        grid.draw(gtable)
+      } else {
+        if (is.character(vp)) seekViewport(vp) else pushViewport(vp)
+        grid.draw(gtable)
+        upViewport()
+      }
+
+      if (isTRUE(getOption("BrailleR.VI")) && rlang::is_installed("BrailleR")) {
+        print(asNamespace("BrailleR")$VI(x))
+      }
+
+      invisible(x)
+    }
+})
+
+# The following extractors and subassignment operators are for a smooth
+# transition and should be deprecated in the release cycle after 4.0.0
+local({
+  S7::method(`[[`, class_gg) <- S7::method(`$`, class_gg) <-
+    function(x, i) {
+      if (!S7::prop_exists(x, i) && S7::prop_exists(x, "meta")) {
+        # This is a trick to bridge a gap between S3 and S7. We're allowing
+        # for arbitrary fields by reading/writing to the 'meta' field when the
+        # index does not point to an actual property.
+        # The proper way to go about this is to implement new fields as properties
+        # of a ggplot subclass.
+        S7::prop(x, "meta")[[i]]
+      } else {
+        `[[`(S7::props(x), i)
+      }
+    }
+  S7::method(`[`, class_gg) <- function(x, i) {
+    `[`(S7::props(x), i)
   }
+})
 
-  if (isTRUE(getOption("BrailleR.VI")) && rlang::is_installed("BrailleR")) {
-    print(asNamespace("BrailleR")$VI(x))
-  }
-
-  invisible(x)
-}
-#' @rdname print.ggplot
-#' @method plot ggplot
 #' @export
-plot.ggplot <- print.ggplot
+`[<-.ggplot2::gg` <- function(x, i, value) {
+  S7::props(x) <- `[<-`(S7::props(x), i, value)
+  x
+}
+
+#' @export
+`$<-.ggplot2::gg` <- function(x, i, value) {
+  if (!S7::prop_exists(x, i) && S7::prop_exists(x, "meta")) {
+    # See explanation in accessor
+    S7::prop(x, "meta")[[i]] <- value
+  } else {
+    S7::props(x) <- `[[<-`(S7::props(x), i, value)
+  }
+  x
+}
+
+#' @export
+`[[<-.ggplot2::gg` <- `$<-.ggplot2::gg`
+
+#' @importFrom S7 convert
+# S7 currently attaches the S3 method to the calling environment which gives `ggplot2:::as.list`
+# Wrap in `local()` to provide a temp environment which throws away the attachment
+local({
+  S7::method(convert, list(from = class_ggplot, to = S7::class_list)) <-
+    function(from, to) {
+      S7::props(from)
+    }
+
+  S7::method(as.list, class_ggplot) <-
+    function(x, ...) convert(x, S7::class_list)
+})
