@@ -16,16 +16,24 @@
 #' The documentation on [colour aesthetics][aes_colour_fill_alpha].
 #'
 #' Other alpha scales: [scale_alpha_manual()], [scale_alpha_identity()].
+#'
+#' The `r link_book("alpha scales section", "scales-colour#sec-scales-alpha")`
 #' @export
 #' @examples
 #' p <- ggplot(mpg, aes(displ, hwy)) +
 #'   geom_point(aes(alpha = year))
 #'
+#' # The default range of 0.1-1.0 leaves all data visible
 #' p
+#'
+#' # Include 0 in the range to make data invisible
+#' p + scale_alpha(range = c(0, 1))
+#'
+#' # Changing the title
 #' p + scale_alpha("cylinders")
-#' p + scale_alpha(range = c(0.4, 0.8))
-scale_alpha <- function(name = waiver(), ..., range = c(0.1, 1)) {
-  continuous_scale("alpha", name = name, palette = pal_rescale(range), ...)
+scale_alpha <- function(name = waiver(), ..., range = NULL, aesthetics = "alpha") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
+  continuous_scale(aesthetics, name = name, palette = palette, ...)
 }
 
 #' @rdname scale_alpha
@@ -34,8 +42,9 @@ scale_alpha_continuous <- scale_alpha
 
 #' @rdname scale_alpha
 #' @export
-scale_alpha_binned <- function(name = waiver(), ..., range = c(0.1, 1)) {
-  binned_scale("alpha", name = name, palette = pal_rescale(range), ...)
+scale_alpha_binned <- function(name = waiver(), ..., range = NULL, aesthetics = "alpha") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
+  binned_scale(aesthetics, name = name, palette = palette, ...)
 }
 
 #' @rdname scale_alpha
@@ -49,32 +58,33 @@ scale_alpha_discrete <- function(...) {
 
 #' @rdname scale_alpha
 #' @export
-scale_alpha_ordinal <- function(name = waiver(), ..., range = c(0.1, 1)) {
-  discrete_scale(
-    "alpha", name = name,
-    palette = function(n) seq(range[1], range[2], length.out = n),
-    ...
+scale_alpha_ordinal <- function(name = waiver(), ..., range = NULL, aesthetics = "alpha") {
+  palette <- if (!is.null(range)) {
+    function(n) seq(range[1], range[2], length.out = n)
+  } else {
+    NULL
+  }
+  discrete_scale(aesthetics, name = name, palette = palette, ...)
+}
+
+#' @rdname scale_alpha
+#' @export
+#' @usage NULL
+scale_alpha_datetime <- function(name = waiver(), ..., range = NULL, aesthetics = "alpha") {
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
+  datetime_scale(
+    aesthetics = aesthetics, transform = "time", name = name,
+    palette = palette, ...
   )
 }
 
 #' @rdname scale_alpha
 #' @export
 #' @usage NULL
-scale_alpha_datetime <- function(name = waiver(), ..., range = c(0.1, 1)) {
+scale_alpha_date <- function(name = waiver(), ..., range = NULL, aesthetics = "alpha"){
+  palette <- if (!is.null(range)) pal_rescale(range) else NULL
   datetime_scale(
-    aesthetics = "alpha", transform = "time", name = name,
-    palette = pal_rescale(range),
-    ...
-  )
-}
-
-#' @rdname scale_alpha
-#' @export
-#' @usage NULL
-scale_alpha_date <- function(name = waiver(), ..., range = c(0.1, 1)){
-  datetime_scale(
-    aesthetics = "alpha", transform = "date", name = name,
-    palette = pal_rescale(range),
-    ...
+    aesthetics = aesthetics, transform = "date", name = name,
+    palette = palette, ...
   )
 }
