@@ -10,12 +10,24 @@
 #' level, and increasing by one for each level (i.e. the labels are placed
 #' at integer positions).  This is what allows jittering to work.
 #'
-#' @inheritDotParams discrete_scale
+#' @inheritDotParams discrete_scale -scale_name
 #' @inheritParams discrete_scale
+#' @param palette A palette function that when called with a single integer
+#'   argument (the number of levels in the scale) returns the numerical values
+#'   that they should take.
+#' @param sec.axis [dup_axis()] is used to specify a secondary axis.
+#' @param continuous.limits One of:
+#'   * `NULL` to use the default scale range
+#'   * A numeric vector of length two providing a display range for the scale.
+#'   Use `NA` to refer to the existing minimum or maximum.
+#'   * A function that accepts the limits and returns a numeric vector of
+#'   length two.
 #' @rdname scale_discrete
 #' @family position scales
 #' @seealso
 #' The [position documentation][aes_position].
+#'
+#' The `r link_book("discrete position scales section", "scales-position#sec-discrete-position")`
 #' @export
 #' @examples
 #' ggplot(diamonds, aes(cut)) + geom_bar()
@@ -61,31 +73,37 @@
 #'   geom_point() +
 #'   scale_x_discrete(labels = abbreviate)
 #' }
-scale_x_discrete <- function(name = waiver(), ..., expand = waiver(),
-                             guide = waiver(), position = "bottom") {
+scale_x_discrete <- function(name = waiver(), ..., palette = seq_len,
+                             expand = waiver(), guide = waiver(),
+                             position = "bottom", sec.axis = waiver(),
+                             continuous.limits = NULL) {
   sc <- discrete_scale(
-    aesthetics = c("x", "xmin", "xmax", "xend"), name = name,
-    palette = identity, ...,
+    aesthetics = ggplot_global$x_aes, name = name,
+    palette = palette, ...,
     expand = expand, guide = guide, position = position,
     super = ScaleDiscretePosition
   )
 
   sc$range_c <- ContinuousRange$new()
-  sc
+  sc$continuous_limits <- continuous.limits
+  set_sec_axis(sec.axis, sc)
 }
 #' @rdname scale_discrete
 #' @export
-scale_y_discrete <- function(name = waiver(), ..., expand = waiver(),
-                             guide = waiver(), position = "left") {
+scale_y_discrete <- function(name = waiver(), ..., palette = seq_len,
+                             expand = waiver(), guide = waiver(),
+                             position = "left", sec.axis = waiver(),
+                             continuous.limits = NULL) {
   sc <- discrete_scale(
-    aesthetics = c("y", "ymin", "ymax", "yend"), name = name,
-    palette = identity, ...,
+    aesthetics = ggplot_global$y_aes, name = name,
+    palette = palette, ...,
     expand = expand, guide = guide, position = position,
     super = ScaleDiscretePosition
   )
 
   sc$range_c <- ContinuousRange$new()
-  sc
+  sc$continuous_limits <- continuous.limits
+  set_sec_axis(sec.axis, sc)
 }
 
 # The discrete position scale maintains two separate ranges - one for
@@ -93,13 +111,15 @@ scale_y_discrete <- function(name = waiver(), ..., expand = waiver(),
 # mapping, but makes it possible to place objects at non-integer positions,
 # as is necessary for jittering etc.
 
-#' @rdname ggplot2-ggproto
+#' @rdname Scale
 #' @format NULL
 #' @usage NULL
 #' @export
 ScaleDiscretePosition <- ggproto("ScaleDiscretePosition", ScaleDiscrete,
+  continuous_limits = NULL,
+
   train = function(self, x) {
-    if (is.discrete(x)) {
+    if (is_discrete(x)) {
       self$range$train(x, drop = self$drop, na.rm = !self$na.translate)
     } else {
       self$range_c$train(x)
@@ -122,7 +142,9 @@ ScaleDiscretePosition <- ggproto("ScaleDiscretePosition", ScaleDiscrete,
   },
 
   is_empty = function(self) {
-    is.null(self$range$range) && is.null(self$limits) && is.null(self$range_c$range)
+    is.null(self$range$range) &&
+      (is.null(self$limits) || is.function(self$limits)) &&
+      is.null(self$range_c$range)
   },
 
   reset = function(self) {
@@ -131,8 +153,25 @@ ScaleDiscretePosition <- ggproto("ScaleDiscretePosition", ScaleDiscrete,
   },
 
   map = function(self, x, limits = self$get_limits()) {
-    if (is.discrete(x)) {
-      x <- seq_along(limits)[match(as.character(x), limits)]
+    if (inherits(x, "AsIs")) {
+      return(x)
+    }
+    if (is_discrete(x)) {
+      values <- self$palette(length(limits))
+      if (!is.numeric(values)) {
+        cli::cli_abort(
+          "The {.arg palette} function must return a {.cls numeric} vector.",
+          call = self$call
+        )
+      }
+      if (length(values) < length(limits)) {
+        cli::cli_abort(
+          "The {.arg palette} function must return at least \\
+            {length(limits)} values.",
+          call = self$call
+        )
+      }
+      x <- values[match(as.character(x), limits)]
     }
     mapped_discrete(x)
   },
@@ -145,6 +184,14 @@ ScaleDiscretePosition <- ggproto("ScaleDiscretePosition", ScaleDiscrete,
     expand_limits_scale(self, expand, limits)
   },
 
+  sec_name = function(self) {
+    if (is_waiver(self$secondary.axis)) {
+      waiver()
+    } else {
+      self$secondary.axis$name
+    }
+  },
+
   clone = function(self) {
     new <- ggproto(NULL, self)
     new$range <- DiscreteRange$new()
