@@ -80,17 +80,17 @@ test_that("layout summary - reversed scales", {
   lr <- summarise_layout(ggplot_build(pr))
   expect_equal(lr$xmin, -7.27)
   expect_equal(lr$xmax, -1.33)
-  expect_equal(lr$xscale[[1]]$trans$name, "reverse")
-  expect_equal(lr$xscale[[1]]$trans$transform(5), -5)
+  expect_equal(lr$xscale[[1]]$get_transformation()$name, "reverse")
+  expect_equal(lr$xscale[[1]]$get_transformation()$transform(5), -5)
 })
 
 test_that("layout summary - log scales", {
-  pl <- p + scale_x_log10() + scale_y_continuous(trans = "log2")
+  pl <- p + scale_x_log10() + scale_y_continuous(transform = "log2")
   ll <- summarise_layout(ggplot_build(pl))
-  expect_equal(ll$xscale[[1]]$trans$name, "log-10")
-  expect_equal(ll$xscale[[1]]$trans$transform(100), 2)
-  expect_equal(ll$yscale[[1]]$trans$name, "log-2")
-  expect_equal(ll$yscale[[1]]$trans$transform(16), 4)
+  expect_equal(ll$xscale[[1]]$get_transformation()$name, "log-10")
+  expect_equal(ll$xscale[[1]]$get_transformation()$transform(100), 2)
+  expect_equal(ll$yscale[[1]]$get_transformation()$name, "log-2")
+  expect_equal(ll$yscale[[1]]$get_transformation()$transform(16), 4)
 })
 
 test_that("coord summary - basic", {
@@ -100,7 +100,7 @@ test_that("coord summary - basic", {
 
 test_that("coord summary - log transformations", {
   # Check for coord log transformations (should ignore log scale)
-  pl <- p + scale_x_log10() + coord_trans(x = "log2")
+  pl <- p + scale_x_log10() + coord_transform(x = "log2")
   ll <- summarise_coord(ggplot_build(pl))
   expect_identical(ll, list(xlog = 2, ylog = NA_real_, flip = FALSE))
 })
